#!/usr/bin/env python3

import sys
import argparse
from mpd import MPDClient, CommandError
import random

# Define the list of allowed mood tags
ALLOWED_MOOD_TAGS = {
    "happy",
    "sad",
    "energetic",
    "calm",
    "angry",
    "relaxed",
    "excited",
    "melancholic"
}

def parse_arguments():
    parser = argparse.ArgumentParser(
        description="Add songs with a specified mood tag to the MPD playlist."
    )
    parser.add_argument(
        "mood",
        type=str,
        help=f"Desired mood tag. Allowed values: {', '.join(sorted(ALLOWED_MOOD_TAGS))}"
    )
    parser.add_argument(
        "--host",
        type=str,
        default="localhost",
        help="MPD server host (default: localhost)"
    )
    parser.add_argument(
        "--port",
        type=int,
        default=6600,
        help="MPD server port (default: 6600)"
    )
    return parser.parse_args()

def add_tracks_by_mood(mood, host="localhost", port=6600):
    mood = mood.lower()
    if mood not in ALLOWED_MOOD_TAGS:
        print(f"Error: Invalid mood '{mood}'. Allowed moods are: {', '.join(sorted(ALLOWED_MOOD_TAGS))}.", file=sys.stderr)
        sys.exit(1)

    client = MPDClient()
    try:
        # Connect to the MPD server
        client.connect(host, port)
        print(f"Connected to MPD server at {host}:{port}.")

        # Clear the current playlist
        client.clear()
        print("Cleared the current playlist.")

        # Search for songs with the specified mood tag
        # Assuming 'mood' is stored as a sticker in the format 'mood=<tag>'
        # Using sticker_find to search for songs with the 'mood' sticker
        response = client.sticker_find('song', '', 'mood')

        # Process the response to filter tracks with the specified mood tag
        matching_tracks = []
        for entry in response:
            file_path = entry.get('file')
            sticker = entry.get('sticker')
            if file_path and sticker:
                try:
                    sticker_key, sticker_value = sticker.split('=', 1)
                    if sticker_key == 'mood' and sticker_value.lower() == mood:
                        matching_tracks.append(file_path)
                except ValueError:
                    # Skip malformed sticker entries
                    continue

        if matching_tracks:
            # Remove duplicates
            unique_tracks = sorted(set(matching_tracks))

            # Shuffle songs for randomness
            random.shuffle(unique_tracks)

            # Add unique songs to the playlist
            for track in unique_tracks:
                client.add(track)
            print(f"Added {len(unique_tracks)} unique song(s) with mood '{mood}' to the playlist.")

            # Optionally, save the playlist with a specific name
            # client.save('mood_playlist')

            # Optionally, start playing the playlist
            # client.play()

            sys.exit(0)  # Success
        else:
            print(f"No songs found with mood '{mood}'.")
            sys.exit(0)  # Treat as success since it's not a critical error

    except CommandError as ce:
        print(f"MPD command error: {ce}", file=sys.stderr)
        sys.exit(1)
    except Exception as e:
        print(f"An error occurred: {e}", file=sys.stderr)
        sys.exit(1)
    finally:
        # Close the connection to the MPD server
        try:
            client.close()
            client.disconnect()
            print("Disconnected from MPD server.")
        except Exception:
            pass

def main():
    args = parse_arguments()
    add_tracks_by_mood(args.mood, host=args.host, port=args.port)

if __name__ == "__main__":
    main()

